/*
 * mysql_auth - mysql based authenticator for Squid Proxy
 * 
 * mysql_auth.c
 * (C) 2002 Ervin Hegedus
 * (C) 2008 Andras Miko
 * Released under GPL, see COPYING-2.0 for details.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <mysql/mysql.h>
#include <syslog.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>

#include "define.h"
#include "extern.h"
#include "mysql_auth_md5.h"
#include "mysql_auth_md5crypt.h"

int main ()
{
	MYSQL	  connect;
	MYSQL_RES *result;
	MYSQL_ROW row;

	struct my_params parameters;

	char	user[MAX_STRLEN], password[MAX_STRLEN];
	char	query[MAXLENGTH], input[MAX_STRLEN];
	char	*tstring;

	char	msalt[MAX_STRLEN];
	char	mmagic[MAX_STRLEN];
	char	md5pass[MAX_STRLEN];
	msalt[0] = '\0';
	mmagic[0] = '\0';
	md5pass[0] = '\0';

	setbuf (stdout, NULL);

	/*
	 * try open and read config file, config file place' see define.h
	 */
	if (parse (&parameters) != 0) {

		// question to squid devels: is require print ERR message
		// when some error occur at initialize of authenticator?
		if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
			fprintf (stdout, "ERR error=\"mysql_auth: Can't initialize helper\"\n");
		} else {
			fprintf (stdout, "ERR\n");
		}
		exit (1);
	}

	/*
	 * first, it must to initialize MYSQL variable
	 */
	mysql_library_init (0, NULL, NULL);
	mysql_init (&connect);

	if (&connect == NULL) {
		syslog (LOG_WARNING, "Can't initialize MYSQL structure!");
		if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
			puts ("ERR error=\"mysql_auth: Can't initialize MYSQL structure!\"");
		} else {
			puts ("ERR");
		}
		exit (1);
	}

	/*
	 * try connect to mysql database, with those user/password, what are
	 * defined in mysql_auth.conf - this is the squid -> mysql connect pair!
	 */
	mysql_real_connect (&connect,\
			parameters.var_host_name,\
			parameters.var_user_name,\
			parameters.var_user_password,\
			parameters.var_database_name,\
			0, parameters.var_mysqld_socket, 0);

	/*
	 * if error returns, log trough syslog
	 */
	if (mysql_errno (&connect)) {

		syslog (LOG_WARNING, "Can't connect to mysql server: %s.\n", parameters.var_host_name);
		syslog (LOG_WARNING, "Error was: %d - %s.\n",\
			mysql_errno (&connect), mysql_error (&connect));
		if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
			puts ("ERR error=\"mysql_auth: Can't connect to mysql server - see syslog for more details\"");
		} else {
			puts ("ERR");
		}
		exit (1);
	}

	/*
	 * try to select from table - there is all the same,
	 * what will the result
	 */
	memset (query, 0, MAXLENGTH + 1);

	sprintf (query, "SELECT * FROM %s WHERE %s LIKE 'squid' AND %s LIKE 'squid'", \
		parameters.var_table_name,\
		parameters.var_user_column,\
		parameters.var_password_column);

	mysql_query (&connect, query);

	// error occur, example invalid table name...
	if (mysql_errno (&connect)) {

		syslog (LOG_WARNING, "Can't select from table - error was: %d - %s",\
				mysql_errno (&connect), mysql_error (&connect));
		if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
			puts ("ERR error=\"mysql_auth: Can't select from table - see syslog for more details\"");
		} else {
			puts ("ERR");
		}
		exit (1);
	}

	/*
	 * these are requires for mysql fuctions
	 */
	result = mysql_use_result (&connect);
	while (mysql_fetch_row (result));
	mysql_free_result(result);

	/**********************************
	 * start of auth method
	 **********************************/

	memset (input, 0, MAX_STRLEN);

	while (fgets(input, MAX_STRLEN, stdin)) {

		user[0] = '\0';
		password[0] = '\0';
		msalt[0] = '\0';

		/*
		 * may be this method is too difficult - who knows more about safety of strtok()?
		 */
		tstring = strtok (input, " \n");

		if (tstring == NULL) {	// empty username
			if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
				puts ("ERR error=\"mysql_auth: No username given\"");
			} else {
				puts ("ERR");
			}
			continue;
	    }
		else {
			strcpy (user, tstring);
			tstring = strtok (NULL, " \n"); // password can contain spaces ?? should be: "\n"
			if (tstring == NULL) {	// empty password
				if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
					puts ("ERR error=\"mysql_auth: No password given\"");
				} else {
					puts ("ERR");
				}
				continue;
			}
			else {
				if ((strcasecmp (parameters.var_encrypt_password_form, "yes") == 0) || (strcasecmp (parameters.var_encrypt_password_form, "backend") == 0)) {
					sprintf (password, "password (\"%s\")", tstring);
				}
				else if (strcasecmp (parameters.var_encrypt_password_form, "md5") == 0) {
					char tmp[MAXLENGTH];
					f_md5 (tmp, tstring);
					sprintf (password, "'%s'", tmp);
				}
				else if (strcasecmp (parameters.var_encrypt_password_form, "md5crypt") == 0) {
					memset (query, 0, strlen (query) + 1);
					if (strcasecmp (parameters.var_squidactive_column, "null") == 0) {
						// won't no squidactive field check
						sprintf (query, "SELECT %s AS pass, IF(left(%s, 3) = '$1$', substr(%s, 4, 8), '') AS salt FROM %s WHERE %s LIKE '%s'", \
							parameters.var_password_column,\
							parameters.var_password_column,\
							parameters.var_password_column,\
							parameters.var_table_name,\
							parameters.var_user_column,\
							user);
					} else {
						// will some squidactive field check
						sprintf (query, "SELECT %s AS pass, IF(left(%s, 3) = '$1$', substr(%s, 4, 8), '') AS salt FROM %s WHERE %s LIKE '%s' AND %s = 1", \
							parameters.var_password_column,\
							parameters.var_password_column,\
							parameters.var_password_column,\
							parameters.var_table_name,\
							parameters.var_user_column,\
							user,\
							parameters.var_squidactive_column);
					}
					mysql_query (&connect, query);
					if (mysql_errno (&connect) && mysql_errno (&connect) != 2006) {
						syslog (LOG_WARNING, "Can't select from table - error was: %d - %s",\
							mysql_errno (&connect), mysql_error (&connect));
						if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
							puts ("ERR error=\"mysql_auth: Can't select from table for md5crypt salt - see syslog for more details\"");
						} else {
							puts ("ERR");
						}
						continue;
					}
					else {
						if (mysql_errno (&connect) == 2006) {
							syslog (LOG_WARNING, "Can't select from table - error was: %d - %s",\
								mysql_errno (&connect), mysql_error (&connect));
							/*
							 * try connect to mysql database, with those user/password, what are
							 * defined in mysql_auth.conf - this is the squid -> mysql connect pair!
							 */
							if (mysql_real_connect (&connect,\
									parameters.var_host_name,\
									parameters.var_user_name,\
									parameters.var_user_password,\
									parameters.var_database_name,\
									0, parameters.var_mysqld_socket, 0)) {
								syslog (LOG_NOTICE, "Reconnected to mysql instance, server came back from sightseeing");
								mysql_query (&connect, query);
							}
							else {
								syslog (LOG_WARNING, "Can't reconnect to mysql instance: server remains gone away");
								if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
									puts ("ERR error=\"mysql_auth: Can't reconnect to mysql instance, server remains gone away at md5crypt salt\"");
								} else {
									puts ("ERR");
								}
								continue;
							}
						}
						result = mysql_use_result (&connect);
						while ((row = mysql_fetch_row (result))) {
							strcpy (msalt, (char *)row[1]);
						}
						if (! mysql_num_rows (result) == 1) {
							syslog (LOG_WARNING, "%s login failed..", user);
						}
						mysql_free_result(result);
					}
					md5crypt (md5pass, tstring, msalt, mmagic);
					sprintf (password, "'%s'", md5pass);
				}
				else {
					sprintf (password, "'%s'", tstring);
				}
			}
	    }
		memset (query, 0, strlen (query) + 1);

		if (strcasecmp (parameters.var_squidactive_column, "null") == 0) {
			// won't no squidactive field check
			sprintf (query, "SELECT * FROM %s WHERE %s LIKE '%s' AND %s LIKE %s", \
				parameters.var_table_name,\
				parameters.var_user_column, user,\
				parameters.var_password_column, password);
		} else {
			// will some squidactive field check
			sprintf (query, "SELECT * FROM %s WHERE %s LIKE '%s' AND %s LIKE %s AND %s = 1", \
				parameters.var_table_name,\
				parameters.var_user_column, user,\
				parameters.var_password_column, password,\
				parameters.var_squidactive_column);
		}

		mysql_query (&connect, query);

		/*
		 * may be when mysql_auth runs, meantime lost
		 * connection,
		 * mysql admin change permission, etc...
		 * in this case don't will exit, just return ERR
		 * message
		 */
		if (mysql_errno (&connect) && mysql_errno (&connect) != 2006) {
			syslog (LOG_WARNING, "Can't select from table - error was: %d - %s",\
				mysql_errno (&connect), mysql_error (&connect));
			if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
				puts ("ERR error=\"mysql_auth: Can't select from table for auth - see syslog for more details\"");
			} else {
				puts ("ERR");
			}
			continue;
		}
		else {
			if (mysql_errno (&connect) == 2006) {
				syslog (LOG_WARNING, "Can't select from table - error was: %d - %s",\
					mysql_errno (&connect), mysql_error (&connect));
				/*
				 * try connect to mysql database, with those user/password, what are
				 * defined in mysql_auth.conf - this is the squid -> mysql connect pair!
				 */
				if (mysql_real_connect (&connect, parameters.var_host_name, parameters.var_user_name,\
						parameters.var_user_password, parameters.var_database_name,\
						0, parameters.var_mysqld_socket, 0)) {
					syslog (LOG_NOTICE, "Reconnected to mysql instance, server went just sightseeing");
					mysql_query (&connect, query);
				}
				else {
					syslog (LOG_WARNING, "Can't reconnect to mysql instance: server remains gone away");
					if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
						puts ("ERR error=\"mysql_auth: Can't reconnect to mysql instance, server remains gone away\"");
					} else {
						puts ("ERR");
					}
					continue;
				}
			}
			result = mysql_use_result (&connect);
			while ((row = mysql_fetch_row (result)));

			/*
			 * the number of result is 1 - this doesn't allow
			 * more than one account!!!
			 */

			if (mysql_num_rows (result) == 1) {
				// save auth time into db, if wanted
				if (strcasecmp (parameters.var_successful_auth_column, "null") != 0) {
					sprintf (query, "UPDATE %s SET %s = now() WHERE %s LIKE '%s'", \
						parameters.var_table_name,\
						parameters.var_successful_auth_column,\
						parameters.var_user_column, user);
					mysql_query (&connect, query);
					if (mysql_errno (&connect) && mysql_errno (&connect) != 2006) {
						syslog (LOG_WARNING, "Can't update table - error was: %d - %s",\
							mysql_errno (&connect), mysql_error (&connect));
						if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
							puts ("ERR error=\"mysql_auth: Can't update last auth timestamp - see syslog for more details\"");
						} else {
							puts ("ERR");
						}
						continue;
					}
				}
				if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
					printf ("OK user=%s\n", user);
				} else {
					puts ("OK");
				}
			}
			else {
				syslog (LOG_WARNING, "%s login failed.", user);
				if (strcasecmp (parameters.var_display_errors, "yes") == 0) {
					puts ("ERR error=\"mysql_auth: Login failed\"");
				} else {
					puts ("ERR");
				}
			}
			mysql_free_result(result);
		}
		memset (input, 0, MAX_STRLEN);
	}
	free (parameters.var_host_name);
	free (parameters.var_user_name);
	free (parameters.var_user_password);
	free (parameters.var_database_name);
	free (parameters.var_mysqld_socket);
	free (parameters.var_table_name);
	free (parameters.var_user_column);
	free (parameters.var_password_column);
	free (parameters.var_squidactive_column);
	free (parameters.var_encrypt_password_form);
	free (parameters.var_display_errors);
	mysql_close (&connect);
	mysql_library_end ();
	return 0;
}

